#--
# Copyright (c) 2010-2012 Michael Berkovich, tr8n.net
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#++
#
#-- Tr8n::TranslatorFollowing Schema Information
#
# Table name: tr8n_translator_following
#
#  id               INTEGER         not null, primary key
#  translator_id    integer         
#  object_id        integer         
#  object_type      varchar(255)    
#  created_at       datetime        
#  updated_at       datetime        
#
# Indexes
#
#  index_tr8n_translator_following_on_translator_id    (translator_id) 
#
#++

class Tr8n::TranslatorFollowing < ActiveRecord::Base
  set_table_name :tr8n_translator_following

  attr_accessible :translator_id, :object_id, :object_type
  attr_accessible :translator, :object

  belongs_to :translator, :class_name => "Tr8n::Translator"   
  belongs_to :object, :polymorphic => true

  def self.find_or_create(translator, object)
    following_for(translator, object) || create(:translator => translator, :object => object)
  end

  def self.following_for(translator, object)
    where("translator_id = ? and object_type = ? and object_id = ?", translator.id, object.class.name, object.id).first
  end
  
end
